@@ -39,23 +39,47 @@ ENV_VAR_MAP = {
 
 
 @builtin_func("GetEnvironmentVariableA")
+@builtin_func("GetEnvironmentVariableW")
 #typespec("DWORD GetEnvironmentVariableA(LPCSTR lpName, LPSTR lpBuffer, DWORD nSize)")
 def get_environment_variable(cpu_context, func_name, func_args):
     """
     Retrieves the contents of the specified variable from the environment block of the calling process.
     """
+    wide = func_name.endswith("W")
     var_name_ptr, buffer_ptr, max_size = func_args
-    var_name = cpu_context.read_data(var_name_ptr).decode("utf8")
+    var_name = cpu_context.read_data(
+        var_name_ptr, data_type=constants.WIDE_STRING if wide else constants.STRING
+    ).decode("utf-16-le" if wide else "utf8")
 
-    # Replace some common enviornment variables, and add %'s for others.
+    # Replace some common environment variables, and add %'s for others.
     # - 1 for null terminator
     name = ENV_VAR_MAP.get(var_name.lower(), f"%{var_name}%")[:max_size - 1]
-    cpu_context.write_data(buffer_ptr, name)
+    cpu_context.write_data(buffer_ptr, name, data_type=constants.WIDE_STRING if wide else constants.STRING)
     logger.debug("Getting environment variable: %s -> %s", var_name, name)
 
     return len(name)
 
 
+@builtin_func("GetCurrentDirectoryA")
+@builtin_func("GetCurrentDirectoryW")
+#typespec("DWORD GetCurrentDirectory(DWORD  nBufferLength, LPTSTR lpBuffer);")
+def get_current_directory(cpu_context, func_name, func_args):
+    """
+    Retrieves the current working directory
+
+    Using value "." to represent the current working directory for this emulator
+    """
+    wide = func_name.endswith("W")
+    max_size, buffer_ptr = func_args
+
+    cwd = u"."[:max_size]
+
+    logger.debug("Writing current working directory %s to 0x%08X", cwd, buffer_ptr)
+    cpu_context.write_data(buffer_ptr, cwd, data_type=constants.WIDE_STRING if wide else constants.STRING)
+
+    return len(cwd)
+
+
 @builtin_func("GetModuleFileNameA")
 @builtin_func("GetModuleFileNameW")
 #typespec("DWORD GetModuleFileNameA_1(HMODULE hModule, LPSTR lpFilename, DWORD nSize);")
@@ -138,6 +162,27 @@ def get_system_directory(cpu_context, func_name, func_args):
     return len(system_dir)
 
 
+@builtin_func("GetTempPathA")
+@builtin_func("GetTempPathW")
+#typespec("DWORD GetTempPathA(DWORD nBufferLength,LPSTR lpBuffer);")
+def get_temp_path(cpu_context, func_name, func_args):
+    """
+    Retrieves the path of the temp directory
+
+    Using unexpanded "%Temp%\" to indicate the temp directory for this emulator
+    """
+    wide = func_name.endswith("W")
+    max_size, buffer_ptr = func_args
+
+    # The returned string ends with a backslash
+    temp_dir = u"%Temp%\\"[:max_size]
+
+    logger.debug("Writing temp directory %s to 0x%08X}", temp_dir, buffer_ptr)
+    cpu_context.write_data(buffer_ptr, temp_dir, data_type=constants.WIDE_STRING if wide else constants.STRING)
+
+    return len(temp_dir)
+
+
 @builtin_func("GetTickCount")
 @builtin_func("GetTickCount64")
 #typespec("DWORD GetTickCount();")
@@ -245,20 +290,27 @@ def get_user_name(cpu_context, func_name, func_args):
 
 
 @builtin_func("CreateProcessA")
+@builtin_func("CreateProcessW")
 #typespec("BOOL CreateProcessA(LPCSTR lpApplicationName, LPSTR lpCommandLine, LPSECURITY_ATTRIBUTES lpProcessAttributes, LPSECURITY_ATTRIBUTES lpThreadAttributes, BOOL bInheritHandles, DWORD dwCreationFlags, LPVOID lpEnvironment, LPCSTR lpCurrentDirectory, LPSTARTUPINFOA lpStartupInfo, LPPROCESS_INFORMATION lpProcessInformation)")
 def create_process(cpu_context, func_name, func_args):
     """
     Create a new process.
     """
+    wide = func_name.endswith("W")
+
     app_ptr = func_args[0]
     cmd_ptr = func_args[1]
 
-    cmd = cpu_context.read_data(cmd_ptr).decode("utf8")
+    cmd = cpu_context.read_data(
+        cmd_ptr, data_type=constants.WIDE_STRING if wide else constants.STRING
+    ).decode("utf-16-le" if wide else "utf8")
     if app_ptr:
-        app = cpu_context.read_data(app_ptr).decode("utf8")
+        app = cpu_context.read_data(
+            app_ptr, data_type=constants.WIDE_STRING if wide else constants.STRING
+        ).decode("utf-16-le" if wide else "utf8")
         cmd = app + " " + cmd
 
-    logger.debug("OpenProcessA: %r", cmd)
+    logger.debug("%s: %r", func_name, cmd)
     cpu_context.actions.append(actions.CommandExecuted(cpu_context.ip, cmd))
 
     return random.randint(wc.MIN_HANDLE, wc.MAX_HANDLE)
@@ -481,4 +533,5 @@ def create_semaphore(cpu_context, func_name, func_args):
     """
     Creates or opens a named or unnamed semaphore object and returns a handle to the object.
     """
+    # TODO: Create high level Semaphore object.
     return random.randint(wc.MIN_HANDLE, wc.MAX_HANDLE)
